@@ -18,7 +18,6 @@
 
 
 import sys
-import os
 import shutil
 import cPickle
 
@@ -32,11 +31,12 @@ from cvs2svn_lib.common import DB_OPEN_NEW
 from cvs2svn_lib.common import DB_OPEN_READ
 from cvs2svn_lib.common import DB_OPEN_WRITE
 from cvs2svn_lib.common import Timestamper
-from cvs2svn_lib.log import Log
+from cvs2svn_lib.sort import sort_file
+from cvs2svn_lib.log import logger
 from cvs2svn_lib.pass_manager import Pass
 from cvs2svn_lib.serializer import PrimedPickleSerializer
 from cvs2svn_lib.artifact_manager import artifact_manager
-from cvs2svn_lib.cvs_file_database import CVSFileDatabase
+from cvs2svn_lib.cvs_path_database import CVSPathDatabase
 from cvs2svn_lib.metadata_database import MetadataDatabase
 from cvs2svn_lib.project import read_projects
 from cvs2svn_lib.project import write_projects
@@ -74,74 +74,14 @@ from cvs2svn_lib.svn_commit import SVNRevisionCommit
 from cvs2svn_lib.openings_closings import SymbolingsLogger
 from cvs2svn_lib.svn_commit_creator import SVNCommitCreator
 from cvs2svn_lib.persistence_manager import PersistenceManager
+from cvs2svn_lib.repository_walker import walk_repository
 from cvs2svn_lib.collect_data import CollectData
-from cvs2svn_lib.process import call_command
 from cvs2svn_lib.check_dependencies_pass \
     import CheckItemStoreDependenciesPass
 from cvs2svn_lib.check_dependencies_pass \
     import CheckIndexedItemStoreDependenciesPass
 
 
-def sort_file(infilename, outfilename, options=[]):
-  """Sort file INFILENAME, storing the results to OUTFILENAME.
-
-  OPTIONS is an optional list of strings that are passed as additional
-  options to the sort command."""
-
-  # GNU sort will sort our dates differently (incorrectly!) if our
-  # LC_ALL is anything but 'C', so if LC_ALL is set, temporarily set
-  # it to 'C'
-  lc_all_tmp = os.environ.get('LC_ALL', None)
-  os.environ['LC_ALL'] = 'C'
-
-  # The -T option to sort has a nice side effect.  The Win32 sort is
-  # case insensitive and cannot be used, and since it does not
-  # understand the -T option and dies if we try to use it, there is no
-  # risk that we use that sort by accident.
-  command = [
-      Ctx().sort_executable,
-      '-T', Ctx().tmpdir
-      ] + options + [
-      infilename
-      ]
-
-  try:
-    # Under Windows, the subprocess module uses the Win32
-    # CreateProcess, which always looks in the Windows system32
-    # directory before it looks in the directories listed in the PATH
-    # environment variable.  Since the Windows sort.exe is in the
-    # system32 directory it will always be chosen.  A simple
-    # workaround is to launch the sort in a shell.  When the shell
-    # (cmd.exe) searches it only examines the directories in the PATH
-    # so putting the directory with GNU sort ahead of the Windows
-    # system32 directory will cause GNU sort to be chosen.
-    call_command(
-        command, stdout=open(outfilename, 'w'), shell=(sys.platform=='win32')
-        )
-  finally:
-    if lc_all_tmp is None:
-      del os.environ['LC_ALL']
-    else:
-      os.environ['LC_ALL'] = lc_all_tmp
-
-  # On some versions of Windows, os.system() does not return an error
-  # if the command fails.  So add little consistency tests here that
-  # the output file was created and has the right size:
-
-  if not os.path.exists(outfilename):
-    raise FatalError('Sort output file missing: %r' % (outfilename,))
-
-  if os.path.getsize(outfilename) != os.path.getsize(infilename):
-    raise FatalError(
-        'Sort input and output file sizes differ:\n'
-        '    %r (%d bytes)\n'
-        '    %r (%d bytes)' % (
-            infilename, os.path.getsize(infilename),
-            outfilename, os.path.getsize(outfilename),
-            )
-        )
-
-
 class CollectRevsPass(Pass):
   """This pass was formerly known as pass1."""
 
@@ -150,17 +90,24 @@ class CollectRevsPass(Pass):
     self._register_temp_file(config.SYMBOL_STATISTICS)
     self._register_temp_file(config.METADATA_INDEX_TABLE)
     self._register_temp_file(config.METADATA_STORE)
-    self._register_temp_file(config.CVS_FILES_DB)
+    self._register_temp_file(config.CVS_PATHS_DB)
     self._register_temp_file(config.CVS_ITEMS_STORE)
-    Ctx().revision_recorder.register_artifacts(self)
 
   def run(self, run_options, stats_keeper):
-    Log().quiet("Examining all CVS ',v' files...")
+    logger.quiet("Examining all CVS ',v' files...")
     Ctx()._projects = {}
-    Ctx()._cvs_file_db = CVSFileDatabase(DB_OPEN_NEW)
-    cd = CollectData(Ctx().revision_recorder, stats_keeper)
+    Ctx()._cvs_path_db = CVSPathDatabase(DB_OPEN_NEW)
+    cd = CollectData(stats_keeper)
+
+    # Key generator for CVSFiles:
+    file_key_generator = KeyGenerator()
+
     for project in run_options.projects:
-      cd.process_project(project)
+      Ctx()._projects[project.id] = project
+      cd.process_project(
+          project,
+          walk_repository(project, file_key_generator, cd.record_fatal_error),
+          )
     run_options.projects = None
 
     fatal_errors = cd.close()
@@ -172,9 +119,9 @@ class CollectRevsPass(Pass):
                            + "\n".join(fatal_errors) + "\n"
                            + "Exited due to fatal error(s).")
 
-    Ctx()._cvs_file_db.close()
+    Ctx()._cvs_path_db.close()
     write_projects(artifact_manager.get_temp_file(config.PROJECTS))
-    Log().quiet("Done")
+    logger.quiet("Done")
 
 
 class CleanMetadataPass(Pass):
@@ -246,17 +193,17 @@ class CleanMetadataPass(Pass):
     try:
       metadata.author = self._get_clean_author(metadata.author)
     except UnicodeError, e:
-      Log().warn('%s: %s' % (warning_prefix, e,))
+      logger.warn('%s: %s' % (warning_prefix, e,))
       self.warnings = True
 
     try:
       metadata.log_msg = self._get_clean_log_msg(metadata.log_msg)
     except UnicodeError, e:
-      Log().warn('%s: %s' % (warning_prefix, e,))
+      logger.warn('%s: %s' % (warning_prefix, e,))
       self.warnings = True
 
   def run(self, run_options, stats_keeper):
-    Log().quiet("Converting metadata to UTF8...")
+    logger.quiet("Converting metadata to UTF8...")
     metadata_db = MetadataDatabase(
         artifact_manager.get_temp_file(config.METADATA_STORE),
         artifact_manager.get_temp_file(config.METADATA_INDEX_TABLE),
@@ -295,7 +242,7 @@ class CleanMetadataPass(Pass):
 
     metadata_clean_db.close()
     metadata_db.close()
-    Log().quiet("Done")
+    logger.quiet("Done")
 
 
 class CollateSymbolsPass(Pass):
@@ -482,7 +429,7 @@ class CollateSymbolsPass(Pass):
 
     del self.symbol_stats
 
-    Log().quiet("Done")
+    logger.quiet("Done")
 
 
 class FilterSymbolsPass(Pass):
@@ -492,57 +439,71 @@ class FilterSymbolsPass(Pass):
   references to the excluded symbols."""
 
   def register_artifacts(self):
-    self._register_temp_file(config.SUMMARY_SERIALIZER)
-    self._register_temp_file(config.CVS_REVS_SUMMARY_DATAFILE)
-    self._register_temp_file(config.CVS_SYMBOLS_SUMMARY_DATAFILE)
+    self._register_temp_file(config.ITEM_SERIALIZER)
+    self._register_temp_file(config.CVS_REVS_DATAFILE)
+    self._register_temp_file(config.CVS_SYMBOLS_DATAFILE)
     self._register_temp_file_needed(config.PROJECTS)
     self._register_temp_file_needed(config.SYMBOL_DB)
-    self._register_temp_file_needed(config.CVS_FILES_DB)
+    self._register_temp_file_needed(config.METADATA_CLEAN_STORE)
+    self._register_temp_file_needed(config.METADATA_CLEAN_INDEX_TABLE)
+    self._register_temp_file_needed(config.CVS_PATHS_DB)
     self._register_temp_file_needed(config.CVS_ITEMS_STORE)
-    Ctx().revision_excluder.register_artifacts(self)
+    Ctx().revision_collector.register_artifacts(self)
 
   def run(self, run_options, stats_keeper):
     Ctx()._projects = read_projects(
         artifact_manager.get_temp_file(config.PROJECTS)
         )
-    Ctx()._cvs_file_db = CVSFileDatabase(DB_OPEN_READ)
+    Ctx()._cvs_path_db = CVSPathDatabase(DB_OPEN_READ)
+    Ctx()._metadata_db = MetadataDatabase(
+        artifact_manager.get_temp_file(config.METADATA_CLEAN_STORE),
+        artifact_manager.get_temp_file(config.METADATA_CLEAN_INDEX_TABLE),
+        DB_OPEN_READ,
+        )
     Ctx()._symbol_db = SymbolDatabase()
     cvs_item_store = OldCVSItemStore(
         artifact_manager.get_temp_file(config.CVS_ITEMS_STORE))
 
     cvs_item_serializer = PrimedPickleSerializer(cvs_item_primer)
-    f = open(artifact_manager.get_temp_file(config.SUMMARY_SERIALIZER), 'wb')
+    f = open(artifact_manager.get_temp_file(config.ITEM_SERIALIZER), 'wb')
     cPickle.dump(cvs_item_serializer, f, -1)
     f.close()
 
     rev_db = NewSortableCVSRevisionDatabase(
-        artifact_manager.get_temp_file(config.CVS_REVS_SUMMARY_DATAFILE),
+        artifact_manager.get_temp_file(config.CVS_REVS_DATAFILE),
         cvs_item_serializer,
         )
 
     symbol_db = NewSortableCVSSymbolDatabase(
-        artifact_manager.get_temp_file(config.CVS_SYMBOLS_SUMMARY_DATAFILE),
+        artifact_manager.get_temp_file(config.CVS_SYMBOLS_DATAFILE),
         cvs_item_serializer,
         )
 
-    revision_excluder = Ctx().revision_excluder
+    revision_collector = Ctx().revision_collector
 
-    Log().quiet("Filtering out excluded symbols and summarizing items...")
+    logger.quiet("Filtering out excluded symbols and summarizing items...")
 
     stats_keeper.reset_cvs_rev_info()
-    revision_excluder.start()
+    revision_collector.start()
 
     # Process the cvs items store one file at a time:
     for cvs_file_items in cvs_item_store.iter_cvs_file_items():
-      Log().verbose(cvs_file_items.cvs_file.filename)
-      cvs_file_items.filter_excluded_symbols(revision_excluder)
+      logger.verbose(cvs_file_items.cvs_file.rcs_path)
+      cvs_file_items.filter_excluded_symbols()
       cvs_file_items.mutate_symbols()
       cvs_file_items.adjust_parents()
       cvs_file_items.refine_symbols()
+      cvs_file_items.determine_revision_properties(
+          Ctx().revision_property_setters
+          )
       cvs_file_items.record_opened_symbols()
       cvs_file_items.record_closed_symbols()
       cvs_file_items.check_link_consistency()
 
+      # Give the revision collector a chance to collect data about the
+      # file:
+      revision_collector.process_file(cvs_file_items)
+
       # Store whatever is left to the new file and update statistics:
       stats_keeper.record_cvs_file(cvs_file_items.cvs_file)
       for cvs_item in cvs_file_items.values():
@@ -557,44 +518,50 @@ class FilterSymbolsPass(Pass):
 
     rev_db.close()
     symbol_db.close()
-    revision_excluder.finish()
+    revision_collector.finish()
     cvs_item_store.close()
     Ctx()._symbol_db.close()
-    Ctx()._cvs_file_db.close()
+    Ctx()._cvs_path_db.close()
 
-    Log().quiet("Done")
+    logger.quiet("Done")
 
 
-class SortRevisionSummaryPass(Pass):
-  """Sort the revision summary file."""
+class SortRevisionsPass(Pass):
+  """Sort the revisions file."""
 
   def register_artifacts(self):
-    self._register_temp_file(config.CVS_REVS_SUMMARY_SORTED_DATAFILE)
-    self._register_temp_file_needed(config.CVS_REVS_SUMMARY_DATAFILE)
+    self._register_temp_file(config.CVS_REVS_SORTED_DATAFILE)
+    self._register_temp_file_needed(config.CVS_REVS_DATAFILE)
 
   def run(self, run_options, stats_keeper):
-    Log().quiet("Sorting CVS revision summaries...")
+    logger.quiet("Sorting CVS revision summaries...")
     sort_file(
-        artifact_manager.get_temp_file(config.CVS_REVS_SUMMARY_DATAFILE),
+        artifact_manager.get_temp_file(config.CVS_REVS_DATAFILE),
         artifact_manager.get_temp_file(
-            config.CVS_REVS_SUMMARY_SORTED_DATAFILE))
-    Log().quiet("Done")
+            config.CVS_REVS_SORTED_DATAFILE
+            ),
+        tempdirs=[Ctx().tmpdir],
+        )
+    logger.quiet("Done")
 
 
-class SortSymbolSummaryPass(Pass):
-  """Sort the symbol summary file."""
+class SortSymbolsPass(Pass):
+  """Sort the symbols file."""
 
   def register_artifacts(self):
-    self._register_temp_file(config.CVS_SYMBOLS_SUMMARY_SORTED_DATAFILE)
-    self._register_temp_file_needed(config.CVS_SYMBOLS_SUMMARY_DATAFILE)
+    self._register_temp_file(config.CVS_SYMBOLS_SORTED_DATAFILE)
+    self._register_temp_file_needed(config.CVS_SYMBOLS_DATAFILE)
 
   def run(self, run_options, stats_keeper):
-    Log().quiet("Sorting CVS symbol summaries...")
+    logger.quiet("Sorting CVS symbol summaries...")
     sort_file(
-        artifact_manager.get_temp_file(config.CVS_SYMBOLS_SUMMARY_DATAFILE),
+        artifact_manager.get_temp_file(config.CVS_SYMBOLS_DATAFILE),
         artifact_manager.get_temp_file(
-            config.CVS_SYMBOLS_SUMMARY_SORTED_DATAFILE))
-    Log().quiet("Done")
+            config.CVS_SYMBOLS_SORTED_DATAFILE
+            ),
+        tempdirs=[Ctx().tmpdir],
+        )
+    logger.quiet("Done")
 
 
 class InitializeChangesetsPass(Pass):
@@ -608,11 +575,11 @@ class InitializeChangesetsPass(Pass):
     self._register_temp_file(config.CVS_ITEMS_SORTED_INDEX_TABLE)
     self._register_temp_file_needed(config.PROJECTS)
     self._register_temp_file_needed(config.SYMBOL_DB)
-    self._register_temp_file_needed(config.CVS_FILES_DB)
-    self._register_temp_file_needed(config.SUMMARY_SERIALIZER)
-    self._register_temp_file_needed(config.CVS_REVS_SUMMARY_SORTED_DATAFILE)
+    self._register_temp_file_needed(config.CVS_PATHS_DB)
+    self._register_temp_file_needed(config.ITEM_SERIALIZER)
+    self._register_temp_file_needed(config.CVS_REVS_SORTED_DATAFILE)
     self._register_temp_file_needed(
-        config.CVS_SYMBOLS_SUMMARY_SORTED_DATAFILE)
+        config.CVS_SYMBOLS_SORTED_DATAFILE)
 
   def get_revision_changesets(self):
     """Generate revision changesets, one at a time.
@@ -627,7 +594,7 @@ class InitializeChangesetsPass(Pass):
 
     db = OldSortableCVSRevisionDatabase(
         artifact_manager.get_temp_file(
-            config.CVS_REVS_SUMMARY_SORTED_DATAFILE
+            config.CVS_REVS_SORTED_DATAFILE
             ),
         self.cvs_item_serializer,
         )
@@ -659,7 +626,7 @@ class InitializeChangesetsPass(Pass):
 
     db = OldSortableCVSSymbolDatabase(
         artifact_manager.get_temp_file(
-            config.CVS_SYMBOLS_SUMMARY_SORTED_DATAFILE
+            config.CVS_SYMBOLS_SORTED_DATAFILE
             ),
         self.cvs_item_serializer,
         )
@@ -805,15 +772,15 @@ class InitializeChangesetsPass(Pass):
           )
 
   def run(self, run_options, stats_keeper):
-    Log().quiet("Creating preliminary commit sets...")
+    logger.quiet("Creating preliminary commit sets...")
 
     Ctx()._projects = read_projects(
         artifact_manager.get_temp_file(config.PROJECTS)
         )
-    Ctx()._cvs_file_db = CVSFileDatabase(DB_OPEN_READ)
+    Ctx()._cvs_path_db = CVSPathDatabase(DB_OPEN_READ)
     Ctx()._symbol_db = SymbolDatabase()
 
-    f = open(artifact_manager.get_temp_file(config.SUMMARY_SERIALIZER), 'rb')
+    f = open(artifact_manager.get_temp_file(config.ITEM_SERIALIZER), 'rb')
     self.cvs_item_serializer = cPickle.load(f)
     f.close()
 
@@ -835,8 +802,8 @@ class InitializeChangesetsPass(Pass):
     self.changeset_key_generator = KeyGenerator()
 
     for (changeset, changeset_items) in self.get_changesets():
-      if Log().is_on(Log.DEBUG):
-        Log().debug(repr(changeset))
+      if logger.is_on(logger.DEBUG):
+        logger.debug(repr(changeset))
       changeset_db.store(changeset)
       for cvs_item in changeset_items:
         self.sorted_cvs_items_db.add(cvs_item)
@@ -846,11 +813,11 @@ class InitializeChangesetsPass(Pass):
     cvs_item_to_changeset_id.close()
     changeset_db.close()
     Ctx()._symbol_db.close()
-    Ctx()._cvs_file_db.close()
+    Ctx()._cvs_path_db.close()
 
     del self.cvs_item_serializer
 
-    Log().quiet("Done")
+    logger.quiet("Done")
 
 
 class ProcessedChangesetLogger:
@@ -858,12 +825,12 @@ class ProcessedChangesetLogger:
     self.processed_changeset_ids = []
 
   def log(self, changeset_id):
-    if Log().is_on(Log.DEBUG):
+    if logger.is_on(logger.DEBUG):
       self.processed_changeset_ids.append(changeset_id)
 
   def flush(self):
     if self.processed_changeset_ids:
-      Log().debug(
+      logger.debug(
           'Consumed changeset ids %s'
           % (', '.join(['%x' % id for id in self.processed_changeset_ids]),))
 
@@ -879,7 +846,7 @@ class BreakRevisionChangesetCyclesPass(Pass):
     self._register_temp_file(config.CVS_ITEM_TO_CHANGESET_REVBROKEN)
     self._register_temp_file_needed(config.PROJECTS)
     self._register_temp_file_needed(config.SYMBOL_DB)
-    self._register_temp_file_needed(config.CVS_FILES_DB)
+    self._register_temp_file_needed(config.CVS_PATHS_DB)
     self._register_temp_file_needed(config.CVS_ITEMS_SORTED_STORE)
     self._register_temp_file_needed(config.CVS_ITEMS_SORTED_INDEX_TABLE)
     self._register_temp_file_needed(config.CHANGESETS_STORE)
@@ -925,8 +892,8 @@ class BreakRevisionChangesetCyclesPass(Pass):
         best_i = i
         best_link = link
 
-    if Log().is_on(Log.DEBUG):
-      Log().debug(
+    if logger.is_on(logger.DEBUG):
+      logger.debug(
           'Breaking cycle %s by breaking node %x' % (
           ' -> '.join(['%x' % node.id for node in (cycle + [cycle[0]])]),
           best_link.changeset.id,))
@@ -939,12 +906,12 @@ class BreakRevisionChangesetCyclesPass(Pass):
       self.changeset_graph.add_new_changeset(changeset)
 
   def run(self, run_options, stats_keeper):
-    Log().quiet("Breaking revision changeset dependency cycles...")
+    logger.quiet("Breaking revision changeset dependency cycles...")
 
     Ctx()._projects = read_projects(
         artifact_manager.get_temp_file(config.PROJECTS)
         )
-    Ctx()._cvs_file_db = CVSFileDatabase(DB_OPEN_READ)
+    Ctx()._cvs_path_db = CVSPathDatabase(DB_OPEN_READ)
     Ctx()._symbol_db = SymbolDatabase()
     Ctx()._cvs_items_db = IndexedCVSItemStore(
         artifact_manager.get_temp_file(config.CVS_ITEMS_SORTED_STORE),
@@ -994,9 +961,9 @@ class BreakRevisionChangesetCyclesPass(Pass):
     self.changeset_graph = None
     Ctx()._cvs_items_db.close()
     Ctx()._symbol_db.close()
-    Ctx()._cvs_file_db.close()
+    Ctx()._cvs_path_db.close()
 
-    Log().quiet("Done")
+    logger.quiet("Done")
 
 
 class RevisionTopologicalSortPass(Pass):
@@ -1009,7 +976,7 @@ class RevisionTopologicalSortPass(Pass):
     self._register_temp_file(config.CHANGESETS_REVSORTED_INDEX)
     self._register_temp_file_needed(config.PROJECTS)
     self._register_temp_file_needed(config.SYMBOL_DB)
-    self._register_temp_file_needed(config.CVS_FILES_DB)
+    self._register_temp_file_needed(config.CVS_PATHS_DB)
     self._register_temp_file_needed(config.CVS_ITEMS_SORTED_STORE)
     self._register_temp_file_needed(config.CVS_ITEMS_SORTED_INDEX_TABLE)
     self._register_temp_file_needed(config.CHANGESETS_REVBROKEN_STORE)
@@ -1065,12 +1032,12 @@ class RevisionTopologicalSortPass(Pass):
     changeset_graph.close()
 
   def run(self, run_options, stats_keeper):
-    Log().quiet("Generating CVSRevisions in commit order...")
+    logger.quiet("Generating CVSRevisions in commit order...")
 
     Ctx()._projects = read_projects(
         artifact_manager.get_temp_file(config.PROJECTS)
         )
-    Ctx()._cvs_file_db = CVSFileDatabase(DB_OPEN_READ)
+    Ctx()._cvs_path_db = CVSPathDatabase(DB_OPEN_READ)
     Ctx()._symbol_db = SymbolDatabase()
     Ctx()._cvs_items_db = IndexedCVSItemStore(
         artifact_manager.get_temp_file(config.CVS_ITEMS_SORTED_STORE),
@@ -1088,9 +1055,9 @@ class RevisionTopologicalSortPass(Pass):
     changesets_revordered_db.close()
     Ctx()._cvs_items_db.close()
     Ctx()._symbol_db.close()
-    Ctx()._cvs_file_db.close()
+    Ctx()._cvs_path_db.close()
 
-    Log().quiet("Done")
+    logger.quiet("Done")
 
 
 class BreakSymbolChangesetCyclesPass(Pass):
@@ -1102,7 +1069,7 @@ class BreakSymbolChangesetCyclesPass(Pass):
     self._register_temp_file(config.CVS_ITEM_TO_CHANGESET_SYMBROKEN)
     self._register_temp_file_needed(config.PROJECTS)
     self._register_temp_file_needed(config.SYMBOL_DB)
-    self._register_temp_file_needed(config.CVS_FILES_DB)
+    self._register_temp_file_needed(config.CVS_PATHS_DB)
     self._register_temp_file_needed(config.CVS_ITEMS_SORTED_STORE)
     self._register_temp_file_needed(config.CVS_ITEMS_SORTED_INDEX_TABLE)
     self._register_temp_file_needed(config.CHANGESETS_REVSORTED_STORE)
@@ -1147,8 +1114,8 @@ class BreakSymbolChangesetCyclesPass(Pass):
         best_i = i
         best_link = link
 
-    if Log().is_on(Log.DEBUG):
-      Log().debug(
+    if logger.is_on(logger.DEBUG):
+      logger.debug(
           'Breaking cycle %s by breaking node %x' % (
           ' -> '.join(['%x' % node.id for node in (cycle + [cycle[0]])]),
           best_link.changeset.id,))
@@ -1161,12 +1128,12 @@ class BreakSymbolChangesetCyclesPass(Pass):
       self.changeset_graph.add_new_changeset(changeset)
 
   def run(self, run_options, stats_keeper):
-    Log().quiet("Breaking symbol changeset dependency cycles...")
+    logger.quiet("Breaking symbol changeset dependency cycles...")
 
     Ctx()._projects = read_projects(
         artifact_manager.get_temp_file(config.PROJECTS)
         )
-    Ctx()._cvs_file_db = CVSFileDatabase(DB_OPEN_READ)
+    Ctx()._cvs_path_db = CVSPathDatabase(DB_OPEN_READ)
     Ctx()._symbol_db = SymbolDatabase()
     Ctx()._cvs_items_db = IndexedCVSItemStore(
         artifact_manager.get_temp_file(config.CVS_ITEMS_SORTED_STORE),
@@ -1216,9 +1183,9 @@ class BreakSymbolChangesetCyclesPass(Pass):
     self.changeset_graph = None
     Ctx()._cvs_items_db.close()
     Ctx()._symbol_db.close()
-    Ctx()._cvs_file_db.close()
+    Ctx()._cvs_path_db.close()
 
-    Log().quiet("Done")
+    logger.quiet("Done")
 
 
 class BreakAllChangesetCyclesPass(Pass):
@@ -1230,7 +1197,7 @@ class BreakAllChangesetCyclesPass(Pass):
     self._register_temp_file(config.CVS_ITEM_TO_CHANGESET_ALLBROKEN)
     self._register_temp_file_needed(config.PROJECTS)
     self._register_temp_file_needed(config.SYMBOL_DB)
-    self._register_temp_file_needed(config.CVS_FILES_DB)
+    self._register_temp_file_needed(config.CVS_PATHS_DB)
     self._register_temp_file_needed(config.CVS_ITEMS_SORTED_STORE)
     self._register_temp_file_needed(config.CVS_ITEMS_SORTED_INDEX_TABLE)
     self._register_temp_file_needed(config.CHANGESETS_SYMBROKEN_STORE)
@@ -1253,7 +1220,7 @@ class BreakAllChangesetCyclesPass(Pass):
   def _split_retrograde_changeset(self, changeset):
     """CHANGESET is retrograde.  Split it into non-retrograde changesets."""
 
-    Log().debug('Breaking retrograde changeset %x' % (changeset.id,))
+    logger.debug('Breaking retrograde changeset %x' % (changeset.id,))
 
     self.changeset_graph.delete_changeset(changeset)
 
@@ -1344,8 +1311,8 @@ class BreakAllChangesetCyclesPass(Pass):
         best_i = i
         best_link = link
 
-    if Log().is_on(Log.DEBUG):
-      Log().debug(
+    if logger.is_on(logger.DEBUG):
+      logger.debug(
           'Breaking segment %s by breaking node %x' % (
           ' -> '.join(['%x' % node.id for node in segment]),
           best_link.changeset.id,))
@@ -1371,8 +1338,8 @@ class BreakAllChangesetCyclesPass(Pass):
     It is not guaranteed that the cycle will be broken by one call to
     this routine, but at least some progress must be made."""
 
-    if Log().is_on(Log.DEBUG):
-      Log().debug(
+    if logger.is_on(logger.DEBUG):
+      logger.debug(
           'Breaking cycle %s' % (
           ' -> '.join(['%x' % changeset.id
                        for changeset in cycle + [cycle[0]]]),))
@@ -1381,12 +1348,12 @@ class BreakAllChangesetCyclesPass(Pass):
     self.break_segment([cycle[-1]] + cycle + [cycle[0]])
 
   def run(self, run_options, stats_keeper):
-    Log().quiet("Breaking CVSSymbol dependency loops...")
+    logger.quiet("Breaking CVSSymbol dependency loops...")
 
     Ctx()._projects = read_projects(
         artifact_manager.get_temp_file(config.PROJECTS)
         )
-    Ctx()._cvs_file_db = CVSFileDatabase(DB_OPEN_READ)
+    Ctx()._cvs_path_db = CVSPathDatabase(DB_OPEN_READ)
     Ctx()._symbol_db = SymbolDatabase()
     Ctx()._cvs_items_db = IndexedCVSItemStore(
         artifact_manager.get_temp_file(config.CVS_ITEMS_SORTED_STORE),
@@ -1476,14 +1443,14 @@ class BreakAllChangesetCyclesPass(Pass):
       id = ordered_changesets[next_ordered_changeset]
       path = self.changeset_graph.search_for_path(id, ordered_changeset_ids)
       if path:
-        if Log().is_on(Log.DEBUG):
-          Log().debug('Breaking path from %s to %s' % (path[0], path[-1],))
+        if logger.is_on(logger.DEBUG):
+          logger.debug('Breaking path from %s to %s' % (path[0], path[-1],))
         self.break_segment(path)
       else:
         # There were no ordered changesets among the reachable
         # predecessors, so do generic cycle-breaking:
-        if Log().is_on(Log.DEBUG):
-          Log().debug(
+        if logger.is_on(logger.DEBUG):
+          logger.debug(
               'Breaking generic cycle found from %s'
               % (self.changeset_db[id],)
               )
@@ -1495,7 +1462,7 @@ class BreakAllChangesetCyclesPass(Pass):
     self.cvs_item_to_changeset_id = None
     self.changeset_db = None
 
-    Log().quiet("Done")
+    logger.quiet("Done")
 
 
 class TopologicalSortPass(Pass):
@@ -1505,7 +1472,7 @@ class TopologicalSortPass(Pass):
     self._register_temp_file(config.CHANGESETS_SORTED_DATAFILE)
     self._register_temp_file_needed(config.PROJECTS)
     self._register_temp_file_needed(config.SYMBOL_DB)
-    self._register_temp_file_needed(config.CVS_FILES_DB)
+    self._register_temp_file_needed(config.CVS_PATHS_DB)
     self._register_temp_file_needed(config.CVS_ITEMS_SORTED_STORE)
     self._register_temp_file_needed(config.CVS_ITEMS_SORTED_INDEX_TABLE)
     self._register_temp_file_needed(config.CHANGESETS_ALLBROKEN_STORE)
@@ -1554,12 +1521,12 @@ class TopologicalSortPass(Pass):
     changeset_graph.close()
 
   def run(self, run_options, stats_keeper):
-    Log().quiet("Generating CVSRevisions in commit order...")
+    logger.quiet("Generating CVSRevisions in commit order...")
 
     Ctx()._projects = read_projects(
         artifact_manager.get_temp_file(config.PROJECTS)
         )
-    Ctx()._cvs_file_db = CVSFileDatabase(DB_OPEN_READ)
+    Ctx()._cvs_path_db = CVSPathDatabase(DB_OPEN_READ)
     Ctx()._symbol_db = SymbolDatabase()
     Ctx()._cvs_items_db = IndexedCVSItemStore(
         artifact_manager.get_temp_file(config.CVS_ITEMS_SORTED_STORE),
@@ -1577,9 +1544,9 @@ class TopologicalSortPass(Pass):
 
     Ctx()._cvs_items_db.close()
     Ctx()._symbol_db.close()
-    Ctx()._cvs_file_db.close()
+    Ctx()._cvs_path_db.close()
 
-    Log().quiet("Done")
+    logger.quiet("Done")
 
 
 class CreateRevsPass(Pass):
@@ -1597,7 +1564,7 @@ class CreateRevsPass(Pass):
     self._register_temp_file(config.CVS_REVS_TO_SVN_REVNUMS)
     self._register_temp_file(config.SYMBOL_OPENINGS_CLOSINGS)
     self._register_temp_file_needed(config.PROJECTS)
-    self._register_temp_file_needed(config.CVS_FILES_DB)
+    self._register_temp_file_needed(config.CVS_PATHS_DB)
     self._register_temp_file_needed(config.CVS_ITEMS_SORTED_STORE)
     self._register_temp_file_needed(config.CVS_ITEMS_SORTED_INDEX_TABLE)
     self._register_temp_file_needed(config.SYMBOL_DB)
@@ -1631,22 +1598,22 @@ class CreateRevsPass(Pass):
   def log_svn_commit(self, svn_commit):
     """Output information about SVN_COMMIT."""
 
-    Log().normal(
+    logger.normal(
         'Creating Subversion r%d (%s)'
         % (svn_commit.revnum, svn_commit.get_description(),)
         )
 
     if isinstance(svn_commit, SVNRevisionCommit):
       for cvs_rev in svn_commit.cvs_revs:
-        Log().verbose(' %s %s' % (cvs_rev.cvs_path, cvs_rev.rev,))
+        logger.verbose(' %s %s' % (cvs_rev.cvs_path, cvs_rev.rev,))
 
   def run(self, run_options, stats_keeper):
-    Log().quiet("Mapping CVS revisions to Subversion commits...")
+    logger.quiet("Mapping CVS revisions to Subversion commits...")
 
     Ctx()._projects = read_projects(
         artifact_manager.get_temp_file(config.PROJECTS)
         )
-    Ctx()._cvs_file_db = CVSFileDatabase(DB_OPEN_READ)
+    Ctx()._cvs_path_db = CVSPathDatabase(DB_OPEN_READ)
     Ctx()._symbol_db = SymbolDatabase()
     Ctx()._cvs_items_db = IndexedCVSItemStore(
         artifact_manager.get_temp_file(config.CVS_ITEMS_SORTED_STORE),
@@ -1669,12 +1636,12 @@ class CreateRevsPass(Pass):
     Ctx()._symbolings_logger.close()
     Ctx()._cvs_items_db.close()
     Ctx()._symbol_db.close()
-    Ctx()._cvs_file_db.close()
+    Ctx()._cvs_path_db.close()
 
-    Log().quiet("Done")
+    logger.quiet("Done")
 
 
-class SortSymbolsPass(Pass):
+class SortSymbolOpeningsClosingsPass(Pass):
   """This pass was formerly known as pass6."""
 
   def register_artifacts(self):
@@ -1682,15 +1649,21 @@ class SortSymbolsPass(Pass):
     self._register_temp_file_needed(config.SYMBOL_OPENINGS_CLOSINGS)
 
   def run(self, run_options, stats_keeper):
-    Log().quiet("Sorting symbolic name source revisions...")
+    logger.quiet("Sorting symbolic name source revisions...")
+
+    def sort_key(line):
+      line = line.split(' ', 2)
+      return (int(line[0], 16), int(line[1]), line[2],)
 
     sort_file(
         artifact_manager.get_temp_file(config.SYMBOL_OPENINGS_CLOSINGS),
         artifact_manager.get_temp_file(
-            config.SYMBOL_OPENINGS_CLOSINGS_SORTED),
-        options=['-k', '1,1', '-k', '2,2n', '-k', '3'],
+            config.SYMBOL_OPENINGS_CLOSINGS_SORTED
+            ),
+        key=sort_key,
+        tempdirs=[Ctx().tmpdir],
         )
-    Log().quiet("Done")
+    logger.quiet("Done")
 
 
 class IndexSymbolsPass(Pass):
@@ -1725,7 +1698,7 @@ class IndexSymbolsPass(Pass):
       id, svn_revnum, ignored = line.split(" ", 2)
       id = int(id, 16)
       if id != old_id:
-        Log().verbose(' ', Ctx()._symbol_db.get_symbol(id).name)
+        logger.verbose(' ', Ctx()._symbol_db.get_symbol(id).name)
         old_id = id
         offsets[id] = fpos
 
@@ -1737,14 +1710,14 @@ class IndexSymbolsPass(Pass):
     offsets_db.close()
 
   def run(self, run_options, stats_keeper):
-    Log().quiet("Determining offsets for all symbolic names...")
+    logger.quiet("Determining offsets for all symbolic names...")
     Ctx()._projects = read_projects(
         artifact_manager.get_temp_file(config.PROJECTS)
         )
     Ctx()._symbol_db = SymbolDatabase()
     self.generate_offsets_for_symbolings()
     Ctx()._symbol_db.close()
-    Log().quiet("Done.")
+    logger.quiet("Done.")
 
 
 class OutputPass(Pass):
@@ -1752,7 +1725,7 @@ class OutputPass(Pass):
 
   def register_artifacts(self):
     self._register_temp_file_needed(config.PROJECTS)
-    self._register_temp_file_needed(config.CVS_FILES_DB)
+    self._register_temp_file_needed(config.CVS_PATHS_DB)
     self._register_temp_file_needed(config.CVS_ITEMS_SORTED_STORE)
     self._register_temp_file_needed(config.CVS_ITEMS_SORTED_INDEX_TABLE)
     self._register_temp_file_needed(config.SYMBOL_DB)
@@ -1763,29 +1736,11 @@ class OutputPass(Pass):
     self._register_temp_file_needed(config.CVS_REVS_TO_SVN_REVNUMS)
     Ctx().output_option.register_artifacts(self)
 
-  def get_svn_commits(self):
-    """Generate the SVNCommits in commit order."""
-
-    persistence_manager = PersistenceManager(DB_OPEN_READ)
-
-    svn_revnum = 1 # The first non-trivial commit
-
-    # Peek at the first revision to find the date to use to initialize
-    # the repository:
-    svn_commit = persistence_manager.get_svn_commit(svn_revnum)
-
-    while svn_commit:
-      yield svn_commit
-      svn_revnum += 1
-      svn_commit = persistence_manager.get_svn_commit(svn_revnum)
-
-    persistence_manager.close()
-
   def run(self, run_options, stats_keeper):
     Ctx()._projects = read_projects(
         artifact_manager.get_temp_file(config.PROJECTS)
         )
-    Ctx()._cvs_file_db = CVSFileDatabase(DB_OPEN_READ)
+    Ctx()._cvs_path_db = CVSPathDatabase(DB_OPEN_READ)
     Ctx()._metadata_db = MetadataDatabase(
         artifact_manager.get_temp_file(config.METADATA_CLEAN_STORE),
         artifact_manager.get_temp_file(config.METADATA_CLEAN_INDEX_TABLE),
@@ -1796,18 +1751,24 @@ class OutputPass(Pass):
         artifact_manager.get_temp_file(config.CVS_ITEMS_SORTED_INDEX_TABLE),
         DB_OPEN_READ)
     Ctx()._symbol_db = SymbolDatabase()
+    Ctx()._persistence_manager = PersistenceManager(DB_OPEN_READ)
 
     Ctx().output_option.setup(stats_keeper.svn_rev_count())
 
-    for svn_commit in self.get_svn_commits():
+    svn_revnum = 1
+    svn_commit = Ctx()._persistence_manager.get_svn_commit(svn_revnum)
+    while svn_commit:
       svn_commit.output(Ctx().output_option)
+      svn_revnum += 1
+      svn_commit = Ctx()._persistence_manager.get_svn_commit(svn_revnum)
 
     Ctx().output_option.cleanup()
+    Ctx()._persistence_manager.close()
 
     Ctx()._symbol_db.close()
     Ctx()._cvs_items_db.close()
     Ctx()._metadata_db.close()
-    Ctx()._cvs_file_db.close()
+    Ctx()._cvs_path_db.close()
 
 
 # The list of passes constituting a run of cvs2svn:
@@ -1817,8 +1778,8 @@ passes = [
     CollateSymbolsPass(),
     #CheckItemStoreDependenciesPass(config.CVS_ITEMS_STORE),
     FilterSymbolsPass(),
-    SortRevisionSummaryPass(),
-    SortSymbolSummaryPass(),
+    SortRevisionsPass(),
+    SortSymbolsPass(),
     InitializeChangesetsPass(),
     #CheckIndexedItemStoreDependenciesPass(
     #    config.CVS_ITEMS_SORTED_STORE,
@@ -1829,7 +1790,7 @@ passes = [
     BreakAllChangesetCyclesPass(),
     TopologicalSortPass(),
     CreateRevsPass(),
-    SortSymbolsPass(),
+    SortSymbolOpeningsClosingsPass(),
     IndexSymbolsPass(),
     OutputPass(),
     ]
